
// This is the canvas that appears in our main application window
// that displays the different visualization views of the data

import java.awt.*;
import java.awt.image.*;
import com.sun.java.swing.*;
import java.util.*;

class AnimationCanvas extends JPanel {
    public final static int PRESSURE = 0;
    public final static int DENSITY = 1;
    public final static int SPEED = 2;
    
    private Font font1;
    private sphclient myparent;
    public int num_frames = -1;
    public int cur_frame = 0;
    public int last_loaded = -1;
    public Image img = null;              // image of the last frame loaded
    
    public AnimationCanvas(Frame parent)
    {
        // constructor
    	font1 = new Font("Monospaced", Font.PLAIN, 10);
        
        myparent = (sphclient) parent;
	}

    public void paint (Graphics g)
    {
        // make sure things are blank to start with
        g.setColor(getBackground());
        g.fillRect(0,0,4000,4000);
        
        // load the image and get the min/max info if we have
        // any frames calculated and we haven't already load it 
        
        if ((num_frames >= 0) && (last_loaded != cur_frame))
        {     
              String str = new String();
              
              str = myparent.network.receiver.get_name(cur_frame);
              
              String ext = new String();
              int attrib = -1;    // which attribute to retrieve the min and max of
                                  // -1 flags that we don't want to get the min/max
              
              if (myparent.view == myparent.POS)
                    ext = new String("pos");              
              else if (myparent.view == myparent.PRESSURE)
              {
                    ext = new String("pres");
                    attrib = PRESSURE;
              }
              else if (myparent.view == myparent.DENSITY) 
              {
                    ext = new String("dens");
                    attrib = DENSITY;
              }
              else if (myparent.view == myparent.VEL) 
              {
                    ext = new String("vel");
                    attrib = SPEED;
              }     
              // we use a third party gif image load class since the one
              // in the java toolkit caches files without checking if they
              // have changed, plus it is very slow compared to this one

              String filename = new String(
                            myparent.network.receiver.output_name + "_" +
                            ext + "." + str + ".gif");               
              try {
                            
                    GifImage gif_array = new GifImage(filename);                    
                    img = createImage(gif_array.get());
                                  
              }
              catch (GifImageException e)
              {
                    System.out.println("Error reading in gif file!");
              }
              
              last_loaded = cur_frame;
              
              // now get our min_max data from this file
              
              if (attrib>=0)
              {
                    float [] min_max = myparent.network.receiver.get_min_max(cur_frame, attrib);                   
                    myparent.min_label.setText("Min = " + min_max[0]);
                    myparent.max_label.setText("Max = " + min_max[1]);
              }
                                                        
        }

        // Now display the image if we've got one
        if (img != null)
            g.drawImage(img, 0, 0, this);                                                        
    }

    public void update()
    {
        repaint();
    }
    
    private int canvas_x(double world_x)
    {
        // convert the passed x world coordinate to a canvas x
        
        Dimension d = getSize();        
        return (int) ((world_x / myparent.world.width) * (d.width-1));
    }

    private int scale_x(double world_x)
    {
        // convert the passed x width to a canvas width
        
        Dimension d = getSize();        
        return (int) ((world_x / myparent.world.width) * d.width);
    }

    private int canvas_y(double world_y)
    {
        // convert the passed y world coordinate to a canvas y
        // note: y coordinate is flipped, java's canvas 0,0 is
        // upper-left, but our simulation assumes lower-left
        
        Dimension d = getSize();        
        return (int) ((d.height-1) - ((world_y / myparent.world.height) * (d.height-1)));
    }    

    private int scale_y(double world_y)
    {
        Dimension d = getSize();        
        return (int) ((world_y / myparent.world.height) * d.height);
    }    

    public void increment_frame_count()
    {

        num_frames++;
            
        // update the animation canvas
        if (num_frames == 0)
            update();       
            
        myparent.frame_scroll.setMaximum(num_frames);    
                    
        myparent.update_animation_controls();        
    }

    // update the canvas based on a new number of frame (we've loaded
    // a new set of files)
    public void update_frame_count(int new_total)
    {
        if (new_total < 0)
            myparent.frame_scroll.setMaximum(0);
        else            
            myparent.frame_scroll.setMaximum(new_total);
        
        num_frames = new_total;
        cur_frame = 0;
        last_loaded = -1;
    
        if (num_frames > 0)
            update();
        myparent.update_animation_controls();                    

    }
}