
// This is the canvas that appears on the main application screen that
// shows all the boundary shapes that the user has requested.

import java.awt.*;
import com.sun.java.swing.*;

class BoundaryCanvas extends JPanel {
    private Font font1;
    private sphclient myparent;
    
    public BoundaryCanvas(Frame parent)
    {
        // constructor
    	font1 = new Font("Monospaced", Font.PLAIN, 10);
        
        myparent = (sphclient) parent;
	}

    public void paint (Graphics g)
    {
        // make sure things are blank to start with
        g.setColor(getBackground());
        g.fillRect(0,0,4000,4000);
        
        g.setFont(font1);
        g.setColor(Color.blue);       
               
        for (int i = 0; i<myparent.world.num_shapes; i++)
        {
            draw_shape(myparent.world.shape[i], g);
        }
    }

    public void update()
    {
        repaint();
    }

    private void draw_shape(Shape s, Graphics g)
    {
        // given a shape s, draw it on our canvas
        
        int x,y,r,h,w,x2,y2;
        
        switch (s.type)
        {
            case 0:
                // line                                
                //x = canvas_x(s.upperLeftX);
                //y = canvas_y(s.upperLeftY);
                //x2 = canvas_x(s.lowerRightX);
                //y2 = canvas_y(s.lowerRightY);
    
                //g.drawLine(x, y, x2, y2);
                break;
            case 2:
                // cylinder
                     
                x = canvas_x(s.lowerLeftX);
                y = canvas_y(s.lowerLeftY + s.height);
                w = scale_x(s.width);
                h = scale_y(s.height);
                
                g.fillOval(x, y, w, h);
                break;
            case 1:
                // rectangle      
                
                x = canvas_x(s.lowerLeftX);
                y = canvas_y(s.lowerLeftY + s.height);
                w = scale_x(s.width);
                h = scale_y(s.height);
                
                g.fillRect(x, y, w, h);

                break;
        }
    }
    
    private int canvas_x(double world_x)
    {
        // convert the passed x world coordinate to a canvas x
        
        Dimension d = getSize();        
        return (int) ((world_x / myparent.world.width) * (d.width-1));
    }

    private int scale_x(double world_x)
    {
        // convert the passed x width to a canvas width
        
        Dimension d = getSize();        
        return (int) ((world_x / myparent.world.width) * d.width);
    }

    private int canvas_y(double world_y)
    {
        // convert the passed y world coordinate to a canvas y
        // note: y coordinate is flipped, java's canvas 0,0 is
        // upper-left, but our simulation assumes lower-left
        
        Dimension d = getSize();        
        return (int) ((d.height-1) - ((world_y / myparent.world.height) * (d.height-1)));
    }    

    private int scale_y(double world_y)
    {
        Dimension d = getSize();        
        return (int) ((world_y / myparent.world.height) * d.height);
    }    

}