//***********************************************************
//
// Contains the global simulation parameters
//
// Written by: Keith Vertanen
// Modified by: Brian Schlatter
// Last updated: 6/12/1999
//
//********************************************************** 

#ifndef PARAMETER_HPP
#define PARAMETER_HPP

#include "Point.h"
#include "msgtags.h"

class Parameter {
	public:
		Parameter();
		Parameter(const Parameter & P);
		Parameter & operator = (const Parameter & P);

		friend ostream & operator << (ostream & , const Parameter &);
 
		int numPts;
		double velocityScale;
		Point initVel;
		double initMass;
		double initDensity;
		double initPressure;
		double initPtSpacing;
		double h;
		double c;
		double machNum;
		double betaMax;
		double nu;	
		double bodyFX;
		double bodyFY;
		double densityVariation;
		double lengthScale;
		
		int firstOutput;		// first step to output a frame
		int outputEvery;		// how often to output results to master
		int numSteps;			// how many total steps to run
		double deltaT;			// change in time per time step

		void getParams(double D[PARAM_SIZE]);
		void setParams(double D[PARAM_SIZE]);
		void setParamsClient(double D[PARAM_SIZE_CLIENT]);
};

#endif
