
// Contains all the simulation parameters that the user can set

import java.lang.*;
import java.io.*;

public class Parameters 
{
    
    static int PARAM_SIZE_CLIENT = 15;
       
    double init_velocity_x;
    double init_velocity_y;
    double velocity_scale;
    double mach_number;
    double init_density;
    int num_particles;
    double delta_t;
    double nu;
    int output_every;
    int first_output;
    int num_steps;
    double body_fx;
    double body_fy;
    double density_variation;
    double length_scale;
    
    double[] packet = new double[PARAM_SIZE_CLIENT];      // packet used to send/recv parameters 
    
    public Parameters()
    {   
        // default constructor
        // set all the parameters to their default values
        set_default();
    		//{{INIT_CONTROLS
		//}}
    }
    
    public void update_packet()
    {
        packet[0] = init_velocity_x;
        packet[1] = init_velocity_y;
        packet[2] = velocity_scale;
        packet[3] = mach_number;
        packet[4] = num_particles;
        packet[5] = init_density;
        packet[6] = delta_t;
        packet[7] = nu;
        packet[8] = output_every;
        packet[9] = first_output;
        packet[10] = num_steps;
        packet[11] = body_fx;
        packet[12] = body_fy;
        packet[13] = density_variation;
        packet[14] = length_scale;        
    }
    
    public void set_default()
    {        
        mach_number     = 0.1;
        num_particles   = 1000;
        velocity_scale  = 2.5E-3;
        init_velocity_x = 0.0;
        init_velocity_y = 0.0;
        init_density    = 1000.0;
        delta_t         = 0.0001;
        nu              = 0.001;
        output_every    = 100;
        first_output    = 0;
        num_steps       = 20000;
        body_fx         = 1.0E-3;
        body_fy         = 0.0;
        density_variation = 0.03;
        length_scale      = 1.0E-3;        
    }
    
    // read the sim parameters in from a file
    // returns true if successful, false o.w.
    public boolean read_file(String filename)
    {
        String temp;
        
        DataInputStream input;
        
        try {
              input = new DataInputStream(new FileInputStream(filename));
        } 
        catch (IOException e)
            {
                System.out.println("Error opening "+filename);
                return false;
            }
        try {
                mach_number     = (new Double(input.readUTF())).doubleValue();
                num_particles   = (new Integer(input.readUTF())).intValue();
                velocity_scale  = (new Double(input.readUTF())).doubleValue();
                init_velocity_x = (new Double(input.readUTF())).doubleValue();
                init_velocity_y = (new Double(input.readUTF())).doubleValue();
                init_density    = (new Double(input.readUTF())).doubleValue();
                delta_t         = (new Double(input.readUTF())).doubleValue();
                nu              = (new Double(input.readUTF())).doubleValue();
                output_every    = (new Integer(input.readUTF())).intValue();
                first_output    = (new Integer(input.readUTF())).intValue();
                num_steps       = (new Integer(input.readUTF())).intValue();
                body_fx         = (new Double(input.readUTF())).doubleValue();
                body_fy         = (new Double(input.readUTF())).doubleValue();
                density_variation = (new Double(input.readUTF())).doubleValue();
                length_scale      = (new Double(input.readUTF())).doubleValue();      

        }
        catch (Exception e)
        {
            System.out.println("Error reading data from "+filename);
            return false;
        }        

        try {
            input.close();
        }
        catch (IOException e)
        {
            System.out.println("Error closing "+filename);
            return false;        
        }
        
        return true;
    }
    // write the sim parameters in from a file
    // returns true if successful, false o.w.
    public boolean write_file(String filename)
    {        
        DataOutputStream output;
        
        try {
              output = new DataOutputStream(new FileOutputStream(filename));
        } 
        catch (IOException e)
            {
                System.out.println("Error writing "+filename);
                return false;
            }
        try {
            output.writeUTF((new Double(mach_number).toString()));
            output.writeUTF((new Integer(num_particles).toString()));
            output.writeUTF((new Double(velocity_scale).toString()));
            output.writeUTF((new Double(init_velocity_x).toString()));
            output.writeUTF((new Double(init_velocity_y).toString()));
            output.writeUTF((new Double(init_density).toString()));
            output.writeUTF((new Double(delta_t).toString()));
            output.writeUTF((new Double(nu).toString()));
            output.writeUTF((new Integer(output_every).toString()));
            output.writeUTF((new Integer(first_output).toString()));
            output.writeUTF((new Integer(num_steps).toString()));
            output.writeUTF((new Double(body_fx).toString()));
            output.writeUTF((new Double(body_fy).toString()));
            output.writeUTF((new Double(density_variation).toString()));
            output.writeUTF((new Double(length_scale).toString()));
            
        }
        catch (IOException e)
        {
            System.out.println("Error writing data to "+filename);
            return false;
        }        

        try {
            output.close();
        }
        catch (IOException e)
        {
            System.out.println("Error closing "+filename);
            return false;        
        }
        
        return true;
    }
    
    // these accessor all return the string representing the number
    public String get_num_particles()
    {
        Integer num = new Integer(num_particles);
        return num.toString();
    }
    
    public String get_length_scale()
    {
        Double num = new Double(length_scale);
        return num.toString();
    }
    public String get_velocity_scale()
    {
        Double num = new Double(velocity_scale);
        return num.toString();
    }
 
    public String get_mach_number()
    {
        Double num = new Double(mach_number);
        return num.toString();
    }
    public String get_init_velocity_x()
    {
        Double num = new Double(init_velocity_x);
        return num.toString();
    }
    public String get_init_velocity_y()
    {
        Double num = new Double(init_velocity_y);
        return num.toString();
    }
    public String get_body_fx()
    {
        Double num = new Double(body_fx);
        return num.toString();
    }
    public String get_body_fy()
    {
        Double num = new Double(body_fy);
        return num.toString();
    }
    public String get_density_variation()
    {
        Double num = new Double(density_variation);
        return num.toString();
    }
    public String get_init_density()
    {
        Double num = new Double(init_density);
        return num.toString();
    }
    public String get_delta_t()
    {
        Double num = new Double(delta_t);
        return num.toString();
    }

    public String get_nu()
    {
        Double num = new Double(nu);
        return num.toString();
    }

    public String get_output_every()
    {
        Integer num = new Integer(output_every);
        return num.toString();
    }

    public String get_first_output()
    {
        Integer num = new Integer(first_output);
        return num.toString();
    }

    public String get_num_steps()
    {
        Integer num = new Integer(num_steps);
        return num.toString();
    }

  
	//{{DECLARE_CONTROLS
	//}}
}