// Basic shape class

// The shape class holds the lower left x and y coordinates
// Also has the height and width and type of the shape

// We are assuming the lower left corner of the canvas is 0,0

class Shape implements globals {

    final static int TOTAL_TYPES = 3;    
    final static int BOX = 1;
    final static int CYLINDER = 2;
    
    double lowerLeftX;
    double lowerLeftY;
    double width;
    double height;
    int type;
    
    String names[] = {"Line", "Box", "Cylinder"};

    public Shape()
    {      
        lowerLeftX = 0.0;
        lowerLeftY = 0.0;
        width = 0.0;
        height = 0.0;
        type = 0;
    		//{{INIT_CONTROLS
		//}}
    }

    // construct based on an attribute pack sent from the server
    public Shape(float [] buffer)
    {
        type = (int) buffer[0];       
        
        if (type == CYLINDER)
        {
            // server code sends us the center, we want to get a lower left coordinate
            lowerLeftX = (double) buffer[1] - buffer[3];
            lowerLeftY = (double) buffer[2] - buffer[3];
            width = (double) 2 * buffer[3];
            height = (double) 2 * buffer[3];
        }
        else if (type == BOX)
        {
            lowerLeftX = (double) buffer[1];
            lowerLeftY = (double) buffer[2];
            width = (double) buffer[3];
            height = (double) buffer[4];
        }
        
    }

    // construct based on passed values
    public Shape(double x, double y, double w, double h, int t)
    {        
        lowerLeftX = x;
        lowerLeftY = y;
        width = w;
        height = h;

        type = t;
    }

    public boolean is_inside(float x, float y)
    {
        if (type == BOX)
        {
            if ( (x > lowerLeftX) && (x < lowerLeftX + width) &&
                 ( y > lowerLeftY) && (y < lowerLeftY + height))
                 return true;            
        }
        else if (type == CYLINDER)
        {
            
            if ( ((x - cylinder_x()) * (x - cylinder_x()) + 
                 (y - cylinder_y()) * (y - cylinder_y())) <=                          
                          cylinder_radius() * cylinder_radius())
                 return true;            
        }

        return false;
    }
    
    // set the attributes of this shape
    public void set(double x, double y, double w, double h, int t)
    {
        lowerLeftX = x;
        lowerLeftY = y;
        width = w;
        height = h;

        type = t;
    }
    
    // set the attributes of a cylinder, takes the center coordinate
    // and the radius of the circle    
    public void set_cylinder(double x, double y, double r)
    {
        lowerLeftX = x - r;
        lowerLeftY = y - r;
        width = 2*r;
        height = 2*r;
        type = 2;
    }
    // set the attributes of a rectangle, takes the corners
    public void set_rectangle(double x, double y, double w, double h)
    {
        lowerLeftX = x;
        lowerLeftY = y;
        width = w;
        height = h;        
        type = BOX;
    }
        
    // return an attribute packet appropriate to be sent to the server    
    public double[] get_packet()
    {
        double packet [] = new double[MAX_ATTRS];
        
        if (type == 0)
        {
            // line, we don't implement this on the server side                   
        }
        else if (type == BOX)
        {
            // Box
           
            packet[0] = 1;
            packet[1] = lowerLeftX;
            packet[2] = lowerLeftY;
            packet[3] = width;
            packet[4] = height;            
        }
        else if (type == CYLINDER)
        {
            // Cylinder
            
            packet[0] = 2;
            packet[1] = cylinder_x();
            packet[2] = cylinder_y();
            packet[3] = cylinder_radius();
            packet[4] = -1;                     // not used
        }
    
        return packet;
    }
    
    public String toString()
    {
        return get_str();
    }
    
    // return a human readable string about this shape
    
    public String get_str()
    {
        StringBuffer str;
        int i;
    
        str = new StringBuffer();
        
        str.append(names[type]);

        // pad out to 20 characters
        for (i=0; i<20-str.length(); i++)
            str.append(" ");

        switch (type)
        {
            case 0:
                // line 
                break;
            case CYLINDER: 
                // cylinder
                str.append(" ");
                str.append("(" + cylinder_x() + ", " + cylinder_y() +
                        ")  radius = ");                
                str.append("" + cylinder_radius());
                break;
            case BOX:
                // rectangle
                str.append("(" + lowerLeftX + ", " + lowerLeftY +
                        ") -> (" + (lowerLeftX + width) + ", " + (lowerLeftY + height) + ")");
                break;
                
        }
        
        return "" + str;   
    }

    // return the x coordinate of a cylinder
    public double cylinder_x()
    {
        return lowerLeftX+(width/2.0);
    }
    
    // return the y coordinate of a cylinder
    public double cylinder_y()
    {
        return lowerLeftY+(width/2.0);
    }    
 
    // return the radius of a cylinder
    public double cylinder_radius()
    {
        return width / 2.0;
    }    
     
    // returns the string name of this shape
    public String get_name()
    {
        return names[type];
    }
	//{{DECLARE_CONTROLS
	//}}
}