
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import com.sun.java.swing.*;

// Class that holds all the information from one frame sent from the master

// we have to extend JPanel for some reason in order to use the createImage call (go figure on fish)

public class SimFrame extends JPanel implements globals
{   
    final static boolean COLOR_PLOTS = true;       // true for color plots, false for B&W
    
    public int num_free;
    public int num_bnd;
    public int num_shapes;
    public float container_width;
    public float container_height;
    
    public Particle [] free_particles;
    public Particle [] bnd_particles;
    public Shape [] shapes;
    
    public float min [] = new float[SMOOTH_SIZE_CLIENT];
    public float max [] = new float[SMOOTH_SIZE_CLIENT];
    public float avg [] = new float[SMOOTH_SIZE_CLIENT];
 
    private sphclient myparent;    
    private int message_size;
    private int width, height; 
    private AttributeGrid grid;
    
    public SimFrame(Frame parent, int w, int h)
    {
        myparent = (sphclient) parent;
        
        num_free = 0;
        num_bnd = 0;
        num_shapes = 0;
        container_width = 0.0f;
        container_height = 0.0f;        
        
        width = w;      // this is our screen resolution
        height = h;
    }
 
    // receive a frame over the socket from our server
    public int get_from_server()
    {
          int A [] = new int[1];
          int pos = 0;
          
          try {            
            myparent.network.mylink.recv_ints(A, 1);

            System.out.println("Server is sending us " + A[0] + " floats.");          
            message_size = A[0];          
            
            if (message_size > 0)
            {
                    float buffer[] = new float[message_size];                                                 
                    myparent.network.mylink.recv_floats(buffer, message_size);
          
                    // unpack the attributes at the beginning
                    
                    num_free = (int) buffer[0];
                    num_bnd = (int) buffer[1];
                    container_width = buffer[2];
                    container_height = buffer[3];
                    num_shapes = (int) buffer[4];

System.out.println("Recv'd frame, num_free = " + num_free);
System.out.println("Recv'd frame, num_bnd = " + num_bnd);
System.out.println("Recv'd frame, width = " + container_width);
System.out.println("Recv'd frame, height = " + container_height);
System.out.println("Recv'd frame, num_shapes = " + num_shapes);

                    pos = 5;
                    // allocate memory based on these attributes
                    free_particles = new Particle[num_free];
                    bnd_particles = new Particle[num_bnd];
                    shapes = new Shape[num_shapes];
          
                    int cur_free = 0;
                    int cur_bnd = 0;
                    int cur_shape = 0;
                    
                    // get the min, max and avg info
                    for (int j = 0; j < SMOOTH_SIZE_CLIENT; j++)
                    {
                        min[j] = buffer[pos++];                        
                        max[j] = buffer[pos++];                        
                        avg[j] = buffer[pos++];                        
                    }
          
                    // now add the shapes
                    
                    for (int j = 0; j < num_shapes; j++)
                    {
                        float shape_pack [] = new float[SHAPE_SIZE];
                                               
                        for (int k = 0; k < SHAPE_SIZE; k++)                                               
                            shape_pack[k] = buffer[pos++];
                        
                        shapes[cur_shape++] = new Shape(shape_pack);
                    }
                    
                    // now add the free particles                    
                    for (int j = 0; j < num_free; j++)
                    {
                        float part_pack []= new float[SMOOTH_SIZE_CLIENT];
                                               
                        for (int k = 0; k < SMOOTH_SIZE_CLIENT; k++)                                               
                            part_pack[k] = buffer[pos++];
                        
                        free_particles[cur_free++] = new Particle(part_pack);
                    }
                    
                    // now add the bnd particles                    
                    for (int j = 0; j < num_bnd; j++)
                    {
                        float part_pack [] = new float[SMOOTH_SIZE_CLIENT];
                                               
                        for (int k = 0; k < SMOOTH_SIZE_CLIENT; k++)                                               
                            part_pack[k] = buffer[pos++];
                        
                        bnd_particles[cur_bnd++] = new Particle(part_pack);
                    }
    
                    
                    // build the grid we will use to speed the NN calculations
                    grid = new AttributeGrid(num_free, container_width, container_height);
                    grid.add(free_particles);
                                    
                    return message_size;
            } // end if             
              else
                return -1;      // the server said we are done
          } catch (Exception e)
          {
            System.out.println("Error reading frame from the server.");
          }
          
          return -1;
    }

    // load a frame from disk
    public void read_file(String filename)
    {          
          int pos = 0;
          int file_size = 0;
          
          try {            

            file_size = (int) (new File(filename)).length();
            
            if (file_size > 0)
            {
                
                ByteArrayInputStream bytestream;
                                                          
                BufferedInputStream input = new BufferedInputStream(                
                        new FileInputStream(filename));
                                           
                byte byte_array[] = new byte[file_size];
            
                input.read(byte_array);       
                input.close();
        
                bytestream = new ByteArrayInputStream(byte_array);
                DataInputStream in;
	            in = new DataInputStream(bytestream);
                
                num_free = (int) in.readFloat();                
                num_bnd = (int) in.readFloat();
                container_width = in.readFloat();
                container_height = in.readFloat();
                num_shapes = (int) in.readFloat();
                          
                // allocate memory based on these attributes
                free_particles = new Particle[num_free];
                bnd_particles = new Particle[num_bnd];
                shapes = new Shape[num_shapes];
          
                int cur_free = 0;
                int cur_bnd = 0;
                int cur_shape = 0;
                    
                // get the min, max and avg info
                for (int j = 0; j < SMOOTH_SIZE_CLIENT; j++)
                {
                    min[j] = in.readFloat();                        
                    max[j] = in.readFloat();                        
                    avg[j] = in.readFloat();                        
                }
          
                // now add the shapes
                    
                for (int j = 0; j < num_shapes; j++)
                {
                    float shape_pack [] = new float[SHAPE_SIZE];
                                               
                    for (int k = 0; k < SHAPE_SIZE; k++)                                               
                        shape_pack[k] = in.readFloat();
                        
                    shapes[cur_shape++] = new Shape(shape_pack);
                }
                    
                // now add the free particles                    
                for (int j = 0; j < num_free; j++)
                {
                    float part_pack []= new float[SMOOTH_SIZE_CLIENT];
                                               
                    for (int k = 0; k < SMOOTH_SIZE_CLIENT; k++)                                               
                        part_pack[k] = in.readFloat();
                        
                    free_particles[cur_free++] = new Particle(part_pack);
                }
                    
                // now add the bnd particles                    
                for (int j = 0; j < num_bnd; j++)
                {
                    float part_pack [] = new float[SMOOTH_SIZE_CLIENT];
                                               
                    for (int k = 0; k < SMOOTH_SIZE_CLIENT; k++)                                               
                        part_pack[k] = in.readFloat();
                        
                    bnd_particles[cur_bnd++] = new Particle(part_pack);
                }
    
                    
                // build the grid we will use to speed the NN calculations
                grid = new AttributeGrid(num_free, container_width, container_height);
                grid.add(free_particles);                                    
                
            } // end if                           
          } catch (Exception e)
          {
            System.out.println("Error reading frame from disk.");
          }
    }
    
    // load just the head of the frame from disk
    public void read_header(String filename)
    {          
          int pos = 0;
          int file_size = 0;
          
          try {            

            file_size = (int) (new File(filename)).length();
            
            if (file_size > 0)
            {
                
                // we only will need to read in the beginning bit
                
                file_size = (5 + SMOOTH_SIZE_CLIENT * 8) * 4;
                
                ByteArrayInputStream bytestream;
                                                          
                BufferedInputStream input = new BufferedInputStream(                
                        new FileInputStream(filename));
                                           
                byte byte_array[] = new byte[file_size];
            
                input.read(byte_array);       
                input.close();
        
                bytestream = new ByteArrayInputStream(byte_array);
                DataInputStream in;
	            in = new DataInputStream(bytestream);

                num_free = (int) in.readFloat();
                num_bnd = (int) in.readFloat();

                container_width = in.readFloat();
                container_height = in.readFloat();
                num_shapes = (int) in.readFloat();
          
                // allocate memory based on these attributes
                free_particles = new Particle[num_free];
                bnd_particles = new Particle[num_bnd];
                shapes = new Shape[num_shapes];
          
                int cur_free = 0;
                int cur_bnd = 0;
                int cur_shape = 0;
                    
                // get the min, max and avg info
                for (int j = 0; j < SMOOTH_SIZE_CLIENT; j++)
                {
                    min[j] = in.readFloat();                        
                    max[j] = in.readFloat();                        
                    avg[j] = in.readFloat();                        
                }
          
            }
          } catch (Exception e)
          {
            System.out.println("Error occured reading the header info!");
          }
    }

    // write just the header info to disk, skip the shapes and particles
    
    public void write_header(String filename)
    {           
           try {
                RandomAccessFile out;            
                out = new RandomAccessFile(filename, "rw");

                // not sure if the write methods change the location, so
                // we'll seek between each one
                
                out.seek(0);
                out.writeFloat( (float) num_free);     
                out.seek(4);
                out.writeFloat( (float) num_bnd); 
                out.seek(8);
                out.writeFloat(container_width);
                out.seek(12);
                out.writeFloat(container_height);
                out.seek(16);
                out.writeFloat( (float) num_shapes);     
                
                int pos = 16;
                // output the min, max and avg info
                for (int j = 0; j < SMOOTH_SIZE_CLIENT; j++)
                {
                    pos += 4;
                    out.seek(pos);
                    out.writeFloat(min[j]);
                    pos += 4;
                    out.seek(pos);
                    out.writeFloat(max[j]);
                    pos += 4;
                    out.seek(pos);
                    out.writeFloat(avg[j]);                        
                }
                
                out.close();
           }
           catch (Exception e)
           {
                System.out.println("Error writing frame header!");
           }
    }


    public String toString()
    {
        StringBuffer minmax = new StringBuffer();
        StringBuffer sh = new StringBuffer();
        
        for (int i=0; i < SMOOTH_SIZE_CLIENT; i++)        
            minmax.append("\nmin " + min[i] + "  max " + max[i] + "  avg " + avg[i]);        

        for (int i=0; i < num_shapes; i++)                
            sh.append("\n " + shapes[i]);
            
        return "Shape:\n num_free = " + num_free +
                "\n num_bnd = " + num_bnd +
                "\n container_width = " + container_width +
                "\n container_height = " + container_height +
                "\n num_shapes = " + num_shapes + "\n" +
                minmax + sh + "\n";
    }

    // write this frame to disk
    public void write_file(String filename)
    {           
           try {
                // we need to convert our int and float data to an array of
                // bytes so we can output using the buffered stream
            
                ByteArrayOutputStream bytestream;
                
            	bytestream = new ByteArrayOutputStream(message_size);            	                        
            
            	DataOutputStream out;
	            out = new DataOutputStream(bytestream);
                
                BufferedOutputStream output = new BufferedOutputStream(                
                    new FileOutputStream(filename));
                
                // output everything to disk
                out.writeFloat((float) num_free);     
                out.writeFloat((float) num_bnd); 
                out.writeFloat(container_width);
                out.writeFloat(container_height);
                out.writeFloat((float)num_shapes);     
                
                // output the min, max and avg info
                for (int j = 0; j < SMOOTH_SIZE_CLIENT; j++)
                {
                    out.writeFloat(min[j]);
                    out.writeFloat(max[j]);
                    out.writeFloat(avg[j]);                        
                }
          
                // now output the shapes                    
                for (int j = 0; j < num_shapes; j++)
                {
                        double shape_pack [];                        
                        shape_pack = shapes[j].get_packet();
                                               
                        for (int k = 0; k < SHAPE_SIZE; k++)                                               
                            out.writeFloat((float) shape_pack[k]);
                }
                    
                // now output the free particles                    
                for (int j = 0; j < num_free; j++)
                {
                        float part_pack [];
                        
                        part_pack = free_particles[j].get_packet();                                               
                        
                        for (int k = 0; k < SMOOTH_SIZE_CLIENT; k++)                                               
                            out.writeFloat(part_pack[k]);
                }

                // now output the bnd particles                    
                for (int j = 0; j < num_bnd; j++)
                {
                        float part_pack [];
                        
                        part_pack = bnd_particles[j].get_packet();                                               
                        
                        for (int k = 0; k < SMOOTH_SIZE_CLIENT; k++)                                               
                            out.writeFloat(part_pack[k]);
                }
                
                output.write(bytestream.toByteArray(), 0, bytestream.size());
                output.flush();
                output.close();            
 
                if (bytestream.size() != message_size * 4)
                    System.out.println("Panic!  Something wrong in write_file");
                    
           } catch(IOException e)
           {
                System.out.println("Exception occured during writing of frame file!");
           }                      
    }
    
    // Create an image object representing all the particle positions
    // that the server sent us, write it out to disk
    public void pos_image(String filename)
    {        
        int i, j;
        PixelFrame pix = new PixelFrame(width, height, container_width, container_height);

        pix.clear();
        
        // plot the positions of all the free particles
        for (i = 0; i < num_free; i++)
        {
            pix.add(free_particles[i].pos_x, 
                    free_particles[i].pos_y,
                    255, 255, 255);
        }            

        // plot the positions of all the bound particles
        for (i = 0; i < num_bnd; i++)
        {
            pix.add(bnd_particles[i].pos_x, 
                    bnd_particles[i].pos_y,
                    0, 0, 255);
        }            

        Image img = createImage(new MemoryImageSource
                           (width, height, pix.getArray(), 0, width)); 
            
        try {
                DataOutputStream output = new DataOutputStream(
                    new FileOutputStream(filename));
        
                GifEncoder gif_file = new GifEncoder(img, output);                
                gif_file.encode();                
                output.close();
        } catch(IOException e)
        {
                System.out.println("Exception occured during writing of gif file!");
                e.printStackTrace();
        }                     
    }

    // creates a plot of the specified data
    public void plot_image(String filename, int attrib)
    {        
        int i, j;        
        float mymin = 0.0f;
        float mymax = 0.0f;
        PixelFrame pix = new PixelFrame(width, height, container_width, container_height);
        int x_pix, y_pix;
        
        float scale;
        
        switch (attrib)
        {
            case DENSITY:   mymin = min[5];
                            mymax = max[5];
                            break;
            case PRESSURE:  mymin = min[6];
                            mymax = max[6];
                            break;
            case SPEED:     mymin = min[4];
                            mymax = max[4];
                            break;                            
        }
        
        // prevent divide by zero
        if (mymin!=mymax)
            scale = 254.0f / (mymax  - mymin);
        else
            scale = 0.0f;

        // now go through and color each pixel
        for (x_pix = 0; x_pix < width; x_pix++)
        {        
            for (y_pix = 0; y_pix < height; y_pix++)
            {
                // convert this pixel to a world coordinate
                float x = container_width * x_pix / width;
                float y = container_height * y_pix / height;
                
                // see if this point is inside one of our shapes
                if (is_inside_shape(x, y))
                {   
                    // color black                    
                    pix.addAt(x_pix, y_pix, 0, 0, 0);                                        
                }
                else
                {
    
                    // find an estimate of what the attributes of this "virtual" particle should be

                    // this one does a simple NN
                    Particle p = grid.find_neighbor(x, y);

                    // take a weighted inverse average of the nearest k (or slightly more) particles
//                    Particle p = grid.find_avg_neighbor(x, y);

                    // Make sure we found something (we only wouldn't if
                    // there were no free particle in the grid, oh oh!)
                    if (p!=null)
                    {
                        float value = 0.0f;
                        if (attrib == PRESSURE)
                            value = p.pressure;
                        else if (attrib == DENSITY)
                            value = p.density;
                        else if (attrib == SPEED)
                            value = p.speed;                        
                       
                        int color = (int) ((value - mymin) * scale);
                        int red = color;
                        int blue = 254 - color;

                        if (COLOR_PLOTS)
                            pix.addAt(x_pix, y_pix, red, 0, blue);
                        else
                            pix.addAt(x_pix, y_pix, color, color, color);
                    }
                }
        
                         
            }
        }
        // now create an image and write out to a gif file
        Image img = createImage(new MemoryImageSource
                    (width, height, pix.getArray(), 0, width)); 
            
        try {
            DataOutputStream output = new DataOutputStream(
                new FileOutputStream(filename));
        
            GifEncoder gif_file = new GifEncoder(img, output);                
            gif_file.encode();                
            output.close();
        } catch(IOException e)
        {
            System.out.println("Exception occured during writing of gif file!");
            e.printStackTrace();
        }                             
    }
    
    boolean is_inside_shape(float x, float y)
    {
        for (int i = 0; i < num_shapes; i++)
            if (shapes[i].is_inside(x, y))
                return true;
        return false;       
    }
    
}