#include "SlaveWorld.h"
#include <stdio.h>

//******************************************************************
//******************************************************************
SlaveWorld::SlaveWorld(int sy, int ey, int maxpts, int maxshapes) :
theGrid(NULL), param(), timeSteps(0), startY(sy), endY(ey),
maxPts(maxpts), maxShapePts(maxshapes)
{
}


//******************************************************************
// Destructor
//******************************************************************
SlaveWorld::~SlaveWorld()
{
      delete theGrid;
}


//******************************************************************
// Each slave creates his own complete Grid. The method allocates
// enough room for ALL the free and bound particles. It also decalres
// which rows in the Grid it is responsible for.
//******************************************************************
void SlaveWorld::createWorld()
{
	theGrid = new SlaveGrid(param);
	assert(theGrid != NULL);

	theGrid->setNumShapePts(maxShapePts);
	theGrid->setNumPts(maxPts);
	theGrid->setRangeOfRows(startY, endY);

}

//******************************************************************
// Perform the first time step of the simulation
//******************************************************************
void SlaveWorld::firstLoop(int rank)
{
	// Perform one time step on our grid. For each time step, we must
	// update our SPH particles plus maintain our buffer rows.

 	theGrid->calcPressure();
 	theGrid->calcSPHEqns();
	theGrid->updateParticlesFirstLoop();
	theGrid->deleteBufferRows();
	theGrid->updateGrid();

	// theGrid maintains a copy of the timer for debugging output
	theGrid->debugTimeSteps = timeSteps;   
}

//******************************************************************
// Perform one loop for our simulation 
//******************************************************************
void SlaveWorld::nextLoop(int rank)
{
	// perform one time step on our grid

	theGrid->predictParticles();	
	theGrid->calcPressure();
	theGrid->calcSPHEqns();
	
	theGrid->calcCorrectDeltaT();
	
	theGrid->updateParticles();

        // we want to clear out our buffer if we will exchange particles
        // with other processor in the next time step

        if ((timeSteps+1) % UPDATE_EVERY == 0)
        {
                theGrid->deleteBufferRows();
                theGrid->deleteNorthBound();
                theGrid->deleteSouthBound();
        }

	theGrid->updateGrid();

	timeSteps++;
	theGrid->debugTimeSteps = timeSteps;
}

//******************************************************************
// mutator for the globalID attribute, used to set our initial globalID, after
// that, we take care of it ourselves
//******************************************************************
void SlaveWorld::setGlobalID(long id)
{
	globalID = id;
}
