
// This class will receive update message from the server.

// It is threaded so it can be receiving while the user is doing
// other things in the application

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.image.*;
import com.sun.java.swing.*;
import com.sun.java.swing.JPanel;

public class UpdateReceiver extends Thread implements globals
{
    public int mode = NEWSIM;
    
    public sphclient myparent;
    String output_name;
    public int output_num;
    public int width;
    public int height;
    private float max_x;
    private float max_y;
    private AttributeGrid grid;
    private SimFrame sim_frame;
        
    public UpdateReceiver(Frame parent)
    {
        myparent = (sphclient) parent;
        
        output_name = new String("out");
        output_num = 0;
    }
    
    public void run()
    {
        int A[] = new int[1];
        A[0] = 0;
    
        System.out.println("Receiver thread starting up.");
        
        // we'll save the viewing size so in case it gets changed
        // while we are generating frames, we won't modify the size of
        // our Image objects
        width = myparent.canvas_width;           
        height = myparent.canvas_height;
        max_x = (float) myparent.world.width;        
        max_y = (float) myparent.world.height;
    
        while (true)
        {
            
            if (mode == NEWSIM)
            {
                myparent.menu_rescale.setEnabled(false);
                myparent.menu_runsim.setEnabled(false);
                
                sim_frame = new SimFrame(myparent, width, height);
                
                if (sim_frame.get_from_server() > 0)
                {
                    System.out.println("Got a packet from server");
                        
                    String filename = new String(output_name + "." 
                                    + get_name(output_num) + ".dat");
                    sim_frame.write_file(filename);
    System.out.println("Plotting position...");                
                    // create the position image
                    filename = new String(output_name + "_pos." 
                                    + get_name(output_num) + ".gif");
                    sim_frame.pos_image(filename);
    System.out.println("Plotting density...");                
                    // create the density plot
                    filename = new String(output_name + "_dens." 
                                    + get_name(output_num) + ".gif");                                
                    sim_frame.plot_image(filename, DENSITY);
    System.out.println("Plotting pressure...");                
                    // create the pressure plot
                    filename = new String(output_name + "_pres." 
                                    + get_name(output_num) + ".gif");                                
                    sim_frame.plot_image(filename, PRESSURE);
    System.out.println("Plotting velocity...");                
                    // create the velocity plot
                    filename = new String(output_name + "_vel." 
                                    + get_name(output_num) + ".gif");                                
                    sim_frame.plot_image(filename, SPEED);

                    // tell the animation canvas that we've got a new one
                    myparent.animation_canvas.increment_frame_count();                    
                    
                    output_num++;
                }
                else      
                {
                    myparent.menu_rescale.setEnabled(true);
                    myparent.menu_runsim.setEnabled(true);
                    suspend();
                        
                    // if we get here, we are running another sim                                        

                    // don't reset if we are being woken up to do a rescale
                    if (mode == NEWSIM)
                        output_num = 0;                
                        
                    myparent.animation_canvas.update_frame_count(-1);                
                }
            } // end mode if
            
            else if (mode == RESCALE)
            {
                myparent.menu_runsim.setEnabled(false);
                myparent.menu_rescale.setEnabled(false);                
                rescale();
                myparent.menu_runsim.setEnabled(true);
                myparent.menu_rescale.setEnabled(true);
                suspend();
                
                if (mode == NEWSIM)
                    output_num = 0;                
                    
                myparent.animation_canvas.update_frame_count(-1);
            }            

        }    // end while (true)        
    }
                       
    // return the min and max values for the specified attribute
    public float[] get_min_max(int load_frame, int attrib_num)
    {        
        try 
        {
            String str = get_name(load_frame);
            String filename = new String(
                            myparent.network.receiver.output_name +
                            "." + str + ".dat");               
                            
            DataInputStream input = new DataInputStream(                
                    new FileInputStream(filename));
                    
            float min_max[] = new float[2];

            for (int i = 0; i < 5; i++)         // skip the beginning info
                input.readFloat();
            
            int loop = -1;
            
            // figure out how far the min/max info is in the file
            if (attrib_num == DENSITY)
                loop = 5;
            else if (attrib_num == PRESSURE)                
                loop = 6;
            else if (attrib_num == SPEED)                
                loop = 4;              
            
            for (int i = 0; i <= loop; i++)
            {            
                min_max[0] = input.readFloat();
                min_max[1] = input.readFloat();
                input.readFloat();  // avg
            }            
            
            return min_max;       
        } catch (Exception e)
        {
            System.out.println("Error trying to read min/max data!");
        }
        
        return null;
    }

    String get_name(int i)
    {
           String num_str;        
           if (i < 10)
                num_str = new String ("000" + i);
           else if (i < 100)
                num_str = new String ("00" + i);
           else if (output_num < 1000)
                num_str = new String ("0" + i);
           else
                num_str = new String("" + i);        
           return num_str;        
    }

    // user as specified a new simulation set to work with, change
    // everything that is relavent
    void load_sim(String directory, String filename)
    {
        // first get the base out of the filename
        
        StringBuffer str = new StringBuffer();
        
        int pos = 0;
        
        while ((filename.charAt(pos)!='_') && 
               (filename.charAt(pos)!='.'))
        {
            str.append(filename.charAt(pos));
            pos++;
        }
               
        output_name = directory + str;
        
        // now see how many total frame we have under this name
        
        pos = 0;
        
        File file_test;
        
        do {
            String name = new String(output_name + "." + get_name(pos) + ".dat");
        
            file_test = new File(name);
            pos++;
        } while (file_test.exists());          
        
        if (pos == 0)
        {
            System.out.println("I coudln't find a data set with the base name: " + output_name);
        }
        else
        {
            output_num = pos - 1;    
            myparent.menu_rescale.setEnabled(true);
        }
 
        myparent.animation_canvas.cur_frame = 0;
        myparent.animation_canvas.update_frame_count(output_num-1);       
        
    }

    // rescale the data plots based on global averages instead of local min/max's
    
    void rescale()
    {                
        int num = output_num;

        // reset so our canvas doesn't have anything initially
 
        output_num = 0;
        myparent.animation_canvas.update_frame_count(-1);
        
        // set these (we may not have started the run() method yet)
        width = myparent.canvas_width;           
        height = myparent.canvas_height;
        max_x = (float) myparent.world.width;        
        max_y = (float) myparent.world.height;
  
        // now search for the global min and max of all the local min/max
        
        float global_min [] = new float[SMOOTH_SIZE_CLIENT];
        float global_max [] = new float[SMOOTH_SIZE_CLIENT];

        for (int i=0; i < SMOOTH_SIZE_CLIENT; i++)
        {
            global_min[i] = 999999.0f;
            global_max[i] = -999999.0f;
        }
            
        SimFrame sim_frame;

        for (int i=0; i < num; i++)
        {
            // load this frame into memory
            sim_frame = new SimFrame(myparent, width, height);
            
            String filename = new String(output_name + "." 
                                + get_name(i) + ".dat");          
            
            // to save time, we'll just read the header info and skip
            // the bulk of the frame object
            sim_frame.read_header(filename);
            
            for (int j=0; j < SMOOTH_SIZE_CLIENT; j++)
            {
                if (sim_frame.min[j] < global_min[j])
                    global_min[j] = sim_frame.min[j];
                if (sim_frame.max[j] > global_max[j])
                    global_max[j] = sim_frame.max[j];                    
            }
        }
        
        // now set all the simframes to have new min/max values
        for (int i=0; i < num; i++)
        {
            sim_frame = new SimFrame(myparent, width, height);
            
            String filename = new String(output_name + "." 
                                + get_name(i) + ".dat");                      
            sim_frame.read_header(filename);
            
            for (int j=0; j < SMOOTH_SIZE_CLIENT; j++)
            {
                sim_frame.min[j] = global_min[j];
                sim_frame.max[j] = global_max[j];
            }
            
            // just write over the old header, but leave the rest of the file intact
            sim_frame.write_header(filename);
        }
        
        for (int i=0; i < SMOOTH_SIZE_CLIENT; i++)            
                System.out.println("Global min = " + global_min[i]
                + "  max = " + global_max[i]);            
        
        // finally, write out all the frames and plots
        
        for (int i=0; i < num; i++)
        {                
                // read in the frame from disk (its min/max should now
                // reflect our global min/max)
                
                sim_frame = new SimFrame(myparent, width, height);
            
                String filename = new String(output_name + "." 
                                    + get_name(i) + ".dat");                 
                sim_frame.read_file(filename);
                
System.out.println("Plotting position...");                
                // create the position image
                filename = new String(output_name + "_pos." 
                                + get_name(i) + ".gif");
                sim_frame.pos_image(filename);
System.out.println("Plotting density...");                
                // create the density plot
                filename = new String(output_name + "_dens." 
                                + get_name(i) + ".gif");                                
                sim_frame.plot_image(filename, DENSITY);
System.out.println("Plotting pressure...");                
                // create the pressure plot
                filename = new String(output_name + "_pres." 
                                + get_name(i) + ".gif");                                
                sim_frame.plot_image(filename, PRESSURE);
System.out.println("Plotting velocity...");                
                // create the velocity plot
                filename = new String(output_name + "_vel." 
                                + get_name(i) + ".gif");                                
                sim_frame.plot_image(filename, SPEED);

                // tell the animation canvas that we've got a new one
                myparent.animation_canvas.increment_frame_count();                    
                    
                output_num++;        
        }
    }
}