
// This class contains things relating to the simulation world, the
// shapes, the container width and height

import java.lang.*;
import java.io.*;

public class World
{
    final int MAX_SHAPES = 100;
    
    double height;
    double width;
    Shape shape[] = new Shape[MAX_SHAPES];
    int num_shapes = 0;    
    
    public World()
    {
        // default constructor
        // set all the parameters to their default values
        set_default();
    		//{{INIT_CONTROLS
		//}}
    }

    public boolean is_inside(float x, float y)
    {
        for (int i = 0; i < num_shapes; i++)
            if (shape[i].is_inside(x, y))
                return true;
        return false;
    }

    // get a packet that is to be sent over to the server
    public void get_packet(double [] pack)
    {
        int size = num_shapes * 5 + 2;
        
        if (size>pack.length)
        {
            System.out.println("Array passed to get_packet was too short!");
        }
        
        int index = 0;
        for (int i=0; i<num_shapes; i++)
        {
            pack[index] = shape[i].type;
            index++;
            
            if (shape[i].type == 1)
            {
                // rectangle
                pack[index] = shape[i].lowerLeftX;
                index++;
                pack[index] = shape[i].lowerLeftY;
                index++;    
                pack[index] = shape[i].width;
                index++;
                pack[index] = shape[i].height;
                index++;
            }
            else if (shape[i].type == 2)
            {
                // cylinder
                
                pack[index] = shape[i].lowerLeftX + shape[i].cylinder_radius();
                index++;
                pack[index] = shape[i].lowerLeftY + shape[i].cylinder_radius();
                index++;
                pack[index] = shape[i].cylinder_radius();
                index++;
                pack[index] = -1;   // not used
                index++;
            }
        }
        
        pack[index] = width;
        index++;
        pack[index] = height;
    }
    
    public void set_default()
    {
        height = 1.0;
        width = 1.0;
    }
    
    // remove the ith shape from the world
    public void delete_shape(int i)
    {        
        for (int j=i; j<num_shapes; j++)
            shape[j] = shape[j+1];
        num_shapes--;
    }
    
    // add a cylinder to the world
    public void add_cylinder(double x, double y, double r)
    {
        shape[num_shapes] = new Shape();
        shape[num_shapes].set_cylinder(x,y,r);
        num_shapes++;
    }
    
    // add a rectangle to the world
    public void add_rectangle(double x, double y, double x2, double y2)
    {
        shape[num_shapes] = new Shape();
        shape[num_shapes].set_rectangle(x,y,x2,y2);
        num_shapes++;
    }
        
    // read the sim parameters in from a file
    // returns true if successful, false o.w.
    public boolean read_file(String filename)
    {
        String temp;
        int i, our_num_shapes;
        
        DataInputStream input;

        try {
              input = new DataInputStream(new FileInputStream(filename));
        }
        catch (IOException e)
            {
                System.out.println("Error opening world file: "+filename);
                return false;
            }
        try {
            height = (new Double(input.readUTF())).doubleValue();
            width = (new Double(input.readUTF())).doubleValue();
            our_num_shapes = (new Integer(input.readUTF())).intValue();
            num_shapes = 0;    
            
            // now read in the shapes, if any
            for (i=0; i<our_num_shapes; i++)
            {
                int type;
                double x, y, r, x2, y2;

                type = (new Integer(input.readUTF())).intValue();                
                x = (new Double(input.readUTF())).doubleValue();
                y = (new Double(input.readUTF())).doubleValue();
                if (type == 2)
                {              
                    // shape is a cylinder, only has radius
                    r = (new Double(input.readUTF())).doubleValue();
                    add_cylinder(x, y, r);
                }
                else if (type == 1)
                {
                    // shape is a rectangle
                    x2 = (new Double(input.readUTF())).doubleValue();
                    y2 = (new Double(input.readUTF())).doubleValue();            
                    add_rectangle(x, y, x2, y2);
                }
            }
        }
        catch (Exception e)
        {
            System.out.println("Error reading world data from "+filename);
            return false;
        }

        try {
            input.close();
        }
        catch (IOException e)
        {
            System.out.println("Error closing world file: "+filename);
            return false;
        }

        return true;
    }
    // write the sim parameters in from a file
    // returns true if successful, false o.w.
    public boolean write_file(String filename)
    {
        DataOutputStream output;

        try {
              output = new DataOutputStream(new FileOutputStream(filename));
        }
        catch (IOException e)
            {
                System.out.println("Error writing world file: "+filename);
                return false;
            }
        try {
            output.writeUTF((new Double(height).toString()));;
            output.writeUTF((new Double(width).toString()));
            output.writeUTF((new Integer(num_shapes).toString()));

            // now write the shapes, if any
            for (int i=0; i<num_shapes; i++)
            {
                output.writeUTF((new Integer(shape[i].type).toString()));
                
                if (shape[i].type==2)
                {
                    // cylinder
                    output.writeUTF((new Double(shape[i].cylinder_x()).toString()));
                    output.writeUTF((new Double(shape[i].cylinder_y()).toString()));
                    output.writeUTF((new Double(shape[i].cylinder_radius()).toString()));
                }                    
                else
                {
                    // line or rectangle
                    output.writeUTF((new Double(shape[i].lowerLeftX).toString()));
                    output.writeUTF((new Double(shape[i].lowerLeftY).toString()));
                    output.writeUTF((new Double(shape[i].width).toString()));
                    output.writeUTF((new Double(shape[i].height).toString()));                
                }
            }
        }
        catch (IOException e)
        {
            System.out.println("Error writing world data to "+filename);
            return false;
        }

        try {
            output.close();
        }
        catch (IOException e)
        {
            System.out.println("Error closing world file: "+filename);
            return false;
        }

        return true;
    }

    // these accessor all return the string representing the number

    public String get_height()
    {
        Double num = new Double(height);
        return num.toString();
    }
    public String get_width()
    {
        Double num = new Double(width);
        return num.toString();
    }

    public boolean valid_height(String str)
    {
        double num = (new Double(str).doubleValue());

        if (num>0.0)
            return true;
        else
            return false;
    }

    public boolean valid_width(String str)
    {
        double num = (new Double(str).doubleValue());

        if (num>0.0)
            return true;
        else
            return false;
    }

	//{{DECLARE_CONTROLS
	//}}
}