/*
 
    SPH Simulation Client

    This is control and visualization tool that is used in conjunction
    will the sph server program running on an parallel MPI system.
    
    The client can send the server initial conditions, boundary shapes,
    and simulation parameters.  It will receive periodic updates from
    the server which it will convert to .gif files for display.
    
    The client can also load client format *.dat files that the server
    has produced.  This is done by using the 'Open simulation' menu
    option and selecting one of the *.dat files from the run.  You
    must then select 'Rescale' in order to generate the .gif files for
    visualization.
    
    Keith Vertanen, 6/99
    
*/


import java.awt.*;
import java.awt.event.*;

import com.sun.java.swing.*;

 
public class sphclient extends com.sun.java.swing.JFrame implements globals
{
    // constants that specify which view we have active
    public final static int SHAPE = 0;
    public final static int NETWORK = 1;
    public final static int POS = 2;
    public final static int PRESSURE = 3;
    public final static int DENSITY = 4;
    public final static int VEL = 5;
			
    public Parameters param = new Parameters();
    public World world = new World();
	public BoundaryCanvas shape_canvas = new BoundaryCanvas(this);
    public NetworkCanvas network_canvas = new NetworkCanvas(this);
    public AnimationCanvas animation_canvas = new AnimationCanvas(this);
    public AnimationPlayback playback = new AnimationPlayback(this);
    public int view = SHAPE;
    public Network network = new Network(this);
    public Project project = new Project(this);
    public int canvas_width;
    public int canvas_height;    
    
	public sphclient()
	{      
		
		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		//{{INIT_CONTROLS
		setJMenuBar(mainMenu);
		setTitle("SPH Simulation");
		setDefaultCloseOperation(com.sun.java.swing.JFrame.DO_NOTHING_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout(0,0));
		setSize(505,409);
		setVisible(false);
		saveFileDialog.setMode(FileDialog.SAVE);
		saveFileDialog.setTitle("Save");
		//$$ saveFileDialog.move(24,336);
		openFileDialog.setMode(FileDialog.LOAD);
		openFileDialog.setTitle("Open");
		//$$ openFileDialog.move(0,336);
		JPanel1.setLayout(null);
		getContentPane().add("Center",JPanel1);
		JPanel1.setBackground(java.awt.Color.lightGray);
		JPanel1.setBounds(0,0,505,409);
		label_lower_left.setText("(0.0, 0.0)");
		JPanel1.add(label_lower_left);
		label_lower_left.setForeground(java.awt.Color.black);
		label_lower_left.setFont(new Font("Dialog", Font.BOLD, 10));
		label_lower_left.setBounds(10,285,67,19);
		label_upper_right.setText("(0.0, 0.0)");
		JPanel1.add(label_upper_right);
		label_upper_right.setForeground(java.awt.Color.black);
		label_upper_right.setFont(new Font("Dialog", Font.BOLD, 10));
		label_upper_right.setBounds(320,55,80,18);
		label_view_title.setText("Boundary shapes");
		JPanel1.add(label_view_title);
		label_view_title.setForeground(java.awt.Color.black);
		label_view_title.setBounds(10,10,150,25);
		frame_scroll.setMaximum(0);
		frame_scroll.setOrientation(com.sun.java.swing.JScrollBar.HORIZONTAL);
		frame_scroll.setVisibleAmount(0);
		JPanel1.add(frame_scroll);
		frame_scroll.setBounds(175,345,190,20);
		frame_scroll.setVisible(false);
		frame_label.setText("Frame: 0 / 0");
		JPanel1.add(frame_label);
		frame_label.setForeground(java.awt.Color.black);
		frame_label.setBounds(375,345,75,20);
		frame_label.setVisible(false);
		play_button.setActionCommand("Play");
		JPanel1.add(play_button);
		play_button.setBackground(java.awt.Color.white);
		play_button.setBounds(5,345,60,20);
		play_button.setVisible(false);
		rewind_button.setActionCommand("Rewind");
		JPanel1.add(rewind_button);
		rewind_button.setBackground(java.awt.Color.white);
		rewind_button.setBounds(70,345,64,20);
		rewind_button.setVisible(false);
		stop_button.setActionCommand("Stop");
		JPanel1.add(stop_button);
		stop_button.setBackground(java.awt.Color.white);
		stop_button.setBounds(75,315,65,20);
		stop_button.setVisible(false);
		JPanel1.add(delay_field);
		delay_field.setBounds(230,310,63,19);
		delay_field.setVisible(false);
		delay_label.setText("Delay:");
		JPanel1.add(delay_label);
		delay_label.setForeground(java.awt.Color.black);
		delay_label.setBounds(230,280,56,20);
		delay_label.setVisible(false);
		JPanel1.add(min_label);
		min_label.setForeground(java.awt.Color.blue);
		min_label.setBounds(60,215,60,40);
		min_label.setVisible(false);
		JPanel1.add(max_label);
		max_label.setForeground(java.awt.Color.red);
		max_label.setBounds(195,215,20,40);
		//$$ mainMenu.move(168,312);
		//$$ JMenuBar1.move(168,312);
		fileMenu.setText("File");
		fileMenu.setActionCommand("File");
		fileMenu.setMnemonic((int)'F');
		mainMenu.add(fileMenu);
		open_sim_menu.setText("Open simulation");
		open_sim_menu.setActionCommand("Open simulation");
		open_sim_menu.setMnemonic((int)'O');
		fileMenu.add(open_sim_menu);
		fileMenu.add(JSeparator4);
		editMenu.setText("Edit");
		editMenu.setActionCommand("Edit");
		editMenu.setMnemonic((int)'E');
		mainMenu.add(editMenu);
		JMenuItem1.setText("Parameters");
		JMenuItem1.setActionCommand("Parameters");
		JMenuItem1.setMnemonic((int)'P');
		editMenu.add(JMenuItem1);
		JMenuItem2.setText("World");
		JMenuItem2.setActionCommand("World");
		JMenuItem2.setMnemonic((int)'W');
		editMenu.add(JMenuItem2);
		JMenuItem3.setText("Network");
		JMenuItem3.setActionCommand("Network");
		JMenuItem3.setMnemonic((int)'N');
		editMenu.add(JMenuItem3);
		JMenu1.setText("View");
		JMenu1.setActionCommand("View");
		JMenu1.setMnemonic((int)'V');
		mainMenu.add(JMenu1);
		view_shapes.setText("Boundary shapes");
		view_shapes.setActionCommand("Boundary shapes");
		view_shapes.setSelected(true);
		view_shapes.setMnemonic((int)'B');
		JMenu1.add(view_shapes);
		view_network.setText("Network");
		view_network.setActionCommand("Network");
		view_network.setMnemonic((int)'N');
		JMenu1.add(view_network);
		view_data.setText("Data");
		view_data.setActionCommand("Data");
		view_data.setMnemonic((int)'D');
		JMenu1.add(view_data);
		view_pos.setText("Positions");
		view_pos.setActionCommand("Positions");
		view_pos.setMnemonic((int)'P');
		view_data.add(view_pos);
		view_pressure.setText("Pressure");
		view_pressure.setActionCommand("Pressure");
		view_pressure.setMnemonic((int)'R');
		view_data.add(view_pressure);
		view_density.setText("Density");
		view_density.setActionCommand("Density");
		view_density.setMnemonic((int)'D');
		view_data.add(view_density);
		view_vel.setText("Velocity");
		view_vel.setActionCommand("Velocity");
		view_vel.setMnemonic((int)'V');
		view_data.add(view_vel);
		networkMenu.setText("Network");
		networkMenu.setActionCommand("Network");
		networkMenu.setMnemonic((int)'N');
		mainMenu.add(networkMenu);
		menu_connect.setText("Connect");
		menu_connect.setActionCommand("Connect");
		menu_connect.setMnemonic((int)'C');
		networkMenu.add(menu_connect);
		menu_disconnect.setEnabled(false);
		menu_disconnect.setText("Disconnect");
		menu_disconnect.setActionCommand("Disconnect");
		menu_disconnect.setMnemonic((int)'D');
		networkMenu.add(menu_disconnect);
		menu_shutdown.setEnabled(false);
		menu_shutdown.setText("Shutdown server");
		menu_shutdown.setActionCommand("Shutdown server");
		menu_shutdown.setMnemonic((int)'S');
		networkMenu.add(menu_shutdown);
		networkMenu.add(JSeparator2);
		menu_update.setEnabled(false);
		menu_update.setText("Update stats");
		menu_update.setActionCommand("Update stats");
		menu_update.setMnemonic((int)'U');
		networkMenu.add(menu_update);
		menu_simulation.setText("Simulation");
		menu_simulation.setActionCommand("Simulation");
		menu_simulation.setMnemonic((int)'S');
		mainMenu.add(menu_simulation);
		menu_runsim.setEnabled(false);
		menu_runsim.setText("Run new simulation");
		menu_runsim.setActionCommand("Run test");
		menu_runsim.setMnemonic((int)'R');
		menu_simulation.add(menu_runsim);
		menu_rescale.setEnabled(false);
		menu_rescale.setText("Rescale plots");
		menu_rescale.setActionCommand("Rescale plots");
		menu_rescale.setMnemonic((int)'S');
		menu_simulation.add(menu_rescale);
		helpMenu.setText("Help");
		helpMenu.setActionCommand("Help");
		helpMenu.setMnemonic((int)'H');
		mainMenu.add(helpMenu);
		aboutItem.setHorizontalTextPosition(com.sun.java.swing.SwingConstants.RIGHT);
		aboutItem.setText("About...");
		aboutItem.setActionCommand("About...");
		aboutItem.setMnemonic((int)'A');
		helpMenu.add(aboutItem);
		//$$ imageIcon1.move(0,369);
		menu_open_project.setText("Open project");
		menu_open_project.setActionCommand("Open project");
		menu_open_project.setMnemonic((int)'P');
		fileMenu.add(menu_open_project);
		JMenuItem5.setText("Save project");
		JMenuItem5.setActionCommand("Save project");
		JMenuItem5.setMnemonic((int)'S');
		fileMenu.add(JMenuItem5);
		fileMenu.add(JSeparator1);		
		fileMenu.add(menu_file1);
		menu_file2.setActionCommand("JMenuItem7");
		fileMenu.add(menu_file2);
		menu_file3.setActionCommand("JMenuItem4");
		fileMenu.add(menu_file3);
		fileMenu.add(JSeparator3);
		exitItem.setText("Exit");
		exitItem.setActionCommand("Exit");
		exitItem.setMnemonic((int)'X');
		fileMenu.add(exitItem);
		
		
	
		//}}

		stop_button.setIcon(stopIcon);        
		rewind_button.setIcon(rewindIcon);        
		play_button.setIcon(playIcon);        

		delay_field.setText("" + playback.getDelay());
		
        resize_canvas();				
		JPanel1.add(shape_canvas);	    
		
		network_canvas.setVisible(false);
		JPanel1.add(network_canvas);	    

		animation_canvas.setVisible(false);
		JPanel1.add(animation_canvas);	    
		
		// take care of our dynamic menus
	    update_menus();	    
		
        // shape_canvas.update();
      
        
		//{{INIT_MENUS
		//}}

		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		exitItem.addActionListener(lSymAction);
		aboutItem.addActionListener(lSymAction);
		JMenuItem1.addActionListener(lSymAction);
		SymFocus aSymFocus = new SymFocus();
		JPanel1.addFocusListener(aSymFocus);
		SymComponent aSymComponent = new SymComponent();
		this.addComponentListener(aSymComponent);
		JMenuItem2.addActionListener(lSymAction);
		this.addFocusListener(aSymFocus);
		view_shapes.addActionListener(lSymAction);
		view_network.addActionListener(lSymAction);
		JMenuItem3.addActionListener(lSymAction);
		menu_connect.addActionListener(lSymAction);
		menu_disconnect.addActionListener(lSymAction);
		menu_open_project.addActionListener(lSymAction);
		JMenuItem5.addActionListener(lSymAction);
		menu_shutdown.addActionListener(lSymAction);
		menu_update.addActionListener(lSymAction);
		menu_file1.addActionListener(lSymAction);
		menu_file2.addActionListener(lSymAction);
		menu_file3.addActionListener(lSymAction);
		menu_runsim.addActionListener(lSymAction);
		menu_rescale.addActionListener(lSymAction);
		SymAdjustment lSymAdjustment = new SymAdjustment();
		frame_scroll.addAdjustmentListener(lSymAdjustment);
		play_button.addActionListener(lSymAction);
		rewind_button.addActionListener(lSymAction);
		stop_button.addActionListener(lSymAction);
		delay_field.addActionListener(lSymAction);
		view_pos.addActionListener(lSymAction);
		view_pressure.addActionListener(lSymAction);
		view_density.addActionListener(lSymAction);
		view_vel.addActionListener(lSymAction);
		open_sim_menu.addActionListener(lSymAction);
		//}}

	    
	}

    /**
     * Creates a new instance of JFrame1 with the given title.
     * @param sTitle the title for the new frame.
     * @see #JFrame1()
     */
	public sphclient(String sTitle)
	{
		this();
		setTitle(sTitle);
	}
	
	/**
	 * The entry point for this application.
	 * Sets the Look and Feel to the System Look and Feel.
	 * Creates a new JFrame1 and makes it visible.
	 */
	static public void main(String args[])
	{
		try {
		    // Add the following code if you want the Look and Feel
		    // to be set to the Look and Feel of the native system.
		    /*
		    try {
		        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		    } 
		    catch (Exception e) { 
		    }
		    */

			//Create a new instance of our application's frame, and make it visible.
			(new sphclient()).setVisible(true);
		} 
		catch (Throwable t) {
			t.printStackTrace();
			//Ensure the application exits with an error condition.
			System.exit(1);
		}
	}

    /**
     * Notifies this component that it has been added to a container
     * This method should be called by <code>Container.add</code>, and 
     * not by user code directly.
     * Overridden here to adjust the size of the frame if needed.
     * @see java.awt.Container#removeNotify
     */
	public void addNotify()
	{
		// Record the size of the window prior to calling parents addNotify.
		Dimension size = getSize();
		
		super.addNotify();
		
		if (frameSizeAdjusted)
			return;
		frameSizeAdjusted = true;
		
		// Adjust size of frame according to the insets and menu bar
		com.sun.java.swing.JMenuBar menuBar = getRootPane().getJMenuBar();
		int menuBarHeight = 0;
		if (menuBar != null)
		    menuBarHeight = menuBar.getPreferredSize().height;
		Insets insets = getInsets();
		setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
	}

	// Used by addNotify
	boolean frameSizeAdjusted = false;

	//{{DECLARE_CONTROLS
	java.awt.FileDialog saveFileDialog = new java.awt.FileDialog(this);
	java.awt.FileDialog openFileDialog = new java.awt.FileDialog(this);
	com.sun.java.swing.JPanel JPanel1 = new com.sun.java.swing.JPanel();
	com.sun.java.swing.JLabel label_lower_left = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JLabel label_upper_right = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JLabel label_view_title = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JScrollBar frame_scroll = new com.sun.java.swing.JScrollBar();
	com.sun.java.swing.JLabel frame_label = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JButton play_button = new com.sun.java.swing.JButton();
	com.sun.java.swing.JButton rewind_button = new com.sun.java.swing.JButton();
	com.sun.java.swing.JButton stop_button = new com.sun.java.swing.JButton();
	com.sun.java.swing.JTextField delay_field = new com.sun.java.swing.JTextField();
	com.sun.java.swing.JLabel delay_label = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JLabel min_label = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JLabel max_label = new com.sun.java.swing.JLabel();
	com.sun.java.swing.JMenuBar mainMenu = new com.sun.java.swing.JMenuBar();
	com.sun.java.swing.JMenu fileMenu = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JMenuItem open_sim_menu = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JSeparator JSeparator4 = new com.sun.java.swing.JSeparator();
	com.sun.java.swing.JMenuItem menu_open_project = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem JMenuItem5 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JSeparator JSeparator1 = new com.sun.java.swing.JSeparator();
	com.sun.java.swing.JMenuItem menu_file1 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem menu_file2 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem menu_file3 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JSeparator JSeparator3 = new com.sun.java.swing.JSeparator();
	com.sun.java.swing.JMenuItem exitItem = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenu editMenu = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JMenuItem JMenuItem1 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem JMenuItem2 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem JMenuItem3 = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenu JMenu1 = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JRadioButtonMenuItem view_shapes = new com.sun.java.swing.JRadioButtonMenuItem();
	com.sun.java.swing.JRadioButtonMenuItem view_network = new com.sun.java.swing.JRadioButtonMenuItem();
	com.sun.java.swing.JMenu view_data = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JRadioButtonMenuItem view_pos = new com.sun.java.swing.JRadioButtonMenuItem();
	com.sun.java.swing.JRadioButtonMenuItem view_pressure = new com.sun.java.swing.JRadioButtonMenuItem();
	com.sun.java.swing.JRadioButtonMenuItem view_density = new com.sun.java.swing.JRadioButtonMenuItem();
	com.sun.java.swing.JRadioButtonMenuItem view_vel = new com.sun.java.swing.JRadioButtonMenuItem();
	com.sun.java.swing.JMenu networkMenu = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JMenuItem menu_connect = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem menu_disconnect = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem menu_shutdown = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JSeparator JSeparator2 = new com.sun.java.swing.JSeparator();
	com.sun.java.swing.JMenuItem menu_update = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenu menu_simulation = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JMenuItem menu_runsim = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenuItem menu_rescale = new com.sun.java.swing.JMenuItem();
	com.sun.java.swing.JMenu helpMenu = new com.sun.java.swing.JMenu();
	com.sun.java.swing.JMenuItem aboutItem = new com.sun.java.swing.JMenuItem();
	//}}
    ImageIcon stopIcon = new ImageIcon("Tape_stop.gif");
    ImageIcon rewindIcon = new ImageIcon("Tape_rewind.gif");
    ImageIcon playIcon = new ImageIcon("Tape_play.gif");
    
	//{{DECLARE_MENUS
	//}}

	void exitApplication()
	{
        this.setVisible(false);    // hide the Frame
		this.dispose();            // free the system resources
	    System.exit(0);            // close the application
	}

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowActivated(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == sphclient.this)
				mainFrame_windowActivated(event);
		}

		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == sphclient.this)
				sphclient_windowClosing(event);
		}
	}

	void sphclient_windowClosing(java.awt.event.WindowEvent event)
	{
		// to do: code goes here.
			 
		sphclient_windowClosing_Interaction1(event);
	}

	void sphclient_windowClosing_Interaction1(java.awt.event.WindowEvent event) {

        // Call the same routing that the exit menu does
	    exitItem_actionPerformed_Interaction1();    		
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == exitItem)
				exitItem_actionPerformed(event);
			else if (object == aboutItem)
				aboutItem_actionPerformed(event);
			else if (object == JMenuItem1)
				JMenuItem1_actionPerformed(event);
			else if (object == JMenuItem2)
				JMenuItem2_actionPerformed(event);
			else if (object == view_shapes)
				view_shapes_actionPerformed(event);
			else if (object == view_network)
				view_network_actionPerformed(event);
			else if (object == JMenuItem3)
				JMenuItem3_actionPerformed(event);
			else if (object == menu_connect)
				menuConnect_actionPerformed(event);
			else if (object == menu_disconnect)
				menuDisconnect_actionPerformed(event);
			else if (object == menu_open_project)
				menuOpenProject_actionPerformed(event);
			else if (object == JMenuItem5)
				JMenuItem5_actionPerformed(event);
			else if (object == menu_shutdown)
				menu_shutdown_actionPerformed(event);
			else if (object == menu_update)
				menuUpdate_actionPerformed(event);
			else if (object == menu_file1)
				menuFile1_actionPerformed(event);
			else if (object == menu_file2)
				menuFile2_actionPerformed(event);
			else if (object == menu_file3)
				menuFile3_actionPerformed(event);
			else if (object == menu_runsim)
				menu_runsim_actionPerformed(event);
			if (object == play_button)
				playButton_actionPerformed(event);
			else if (object == rewind_button)
				rewindButton_actionPerformed(event);
			else if (object == stop_button)
				stopButton_actionPerformed(event);
			else if (object == delay_field)
				delayField_actionPerformed(event);
			else if (object == view_pos)
				viewPos_actionPerformed(event);
			else if (object == view_pressure)
				viewPressure_actionPerformed(event);
			else if (object == view_density)
				viewDensity_actionPerformed(event);
			else if (object == view_vel)
				viewVel_actionPerformed(event);
			else if (object == open_sim_menu)
				openSimMenu_actionPerformed(event);
			else if (object == menu_rescale)
				menu_rescale_actionPerformed(event);
			
			
		}
	}

	void openItem_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		openItem_actionPerformed_Interaction1(event);
	}

	void openItem_actionPerformed_Interaction1(java.awt.event.ActionEvent event) {
		try {
			// openFileDialog Show the FileDialog
			openFileDialog.setVisible(true);
		} catch (Exception e) {
		}
	}

	void saveItem_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		saveItem_actionPerformed_Interaction1(event);
	}

	void saveItem_actionPerformed_Interaction1(java.awt.event.ActionEvent event) {
		try {
			// saveFileDialog Show the FileDialog
			saveFileDialog.setVisible(true);
		} catch (Exception e) {
		}
	}

	void exitItem_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		exitItem_actionPerformed_Interaction1();
	}

	void exitItem_actionPerformed_Interaction1() {

        // make sure we are disconnected before we exit
		if (network.connected)		
		    network.disconnect();		

		project.write_last_files();
		
		try {
			this.exitApplication();
		} catch (Exception e) {
		}
	}

	void aboutItem_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		aboutItem_actionPerformed_Interaction1(event);
	}

	void JMenuItem1_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		JMenuItem1_actionPerformed_Interaction1(event);
	}

	void JMenuItem1_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			// edit_parms Create with owner and show as modal
			{
				edit_parms edit_parms1 = new edit_parms(this);
				edit_parms1.setModal(true);
				edit_parms1.show();
			}
		} catch (Exception e) {
		}
	}
	
	void resize_canvas()
    {
        // resize all the elements of all our views
        
        Dimension d = JPanel1.getSize();
        
        Font font = new Font("Dialog", Font.PLAIN, 10);
        
        canvas_width = d.width-40;
        canvas_height = d.height-85;
        
        shape_canvas.setBounds(20, 50, canvas_width, canvas_height);
        network_canvas.setBounds(20, 50, canvas_width, canvas_height);
		animation_canvas.setBounds(20, 50, canvas_width, canvas_height);
		
		label_lower_left.setBounds(10,d.height-25,66,15);
        
		// now do a little song and dance to find out the width of the upper right text
		String str = new String("(" + world.width+", " + world.height+")");
		int strwidth = Toolkit.getDefaultToolkit().getFontMetrics(font).stringWidth(str);
		label_upper_right.setBounds(d.width-strwidth-10,30,strwidth+5,15);        
        label_upper_right.setText(str);

        frame_label.setBounds(d.width - 90, d.height - 25, 110, 15);
        frame_scroll.setBounds(210, d.height - 25, d.width - 95 - 210, 20);

        play_button.setBounds(10, d.height-25, 30, 20);
        stop_button.setBounds(45, d.height-25, 30, 20);
        rewind_button.setBounds(80, d.height-25, 30, 20);
  
        delay_label.setBounds(115, d.height-25, 60, 20);
        delay_field.setBounds(157, d.height-25, 40, 20);
     
        min_label.setBounds(d.width - 275, 30, 150, 15);
        max_label.setBounds(d.width - 140, 30, 150, 15);
        
        update_animation_controls();
    }

	class SymFocus extends java.awt.event.FocusAdapter
	{
		public void focusGained(java.awt.event.FocusEvent event)
		{
			Object object = event.getSource();
			if (object == JPanel1)
				JPanel1_focusGained(event);
			else if (object == sphclient.this)
				mainFrame_focusGained(event);
		}
	}

	void JPanel1_focusGained(java.awt.event.FocusEvent event)
	{
		// to do: code goes here.		
	}

	void mainFrame_windowActivated(java.awt.event.WindowEvent event)
	{
	    // This will fix any bad karma that other windows may have done to us	    
	    this.paintAll(this.getGraphics());	   
	}

	class SymComponent extends java.awt.event.ComponentAdapter
	{
		public void componentResized(java.awt.event.ComponentEvent event)
		{
			Object object = event.getSource();
			if (object == sphclient.this)
				mainFrame_componentResized();
		}
	}

	void mainFrame_componentResized()
	{
	    resize_canvas();
        shape_canvas.update();
        network_canvas.update();  
        animation_canvas.update();
	}


	void JMenuItem2_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		JMenuItem2_actionPerformed_Interaction1(event);
	}

	void JMenuItem2_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			// edit_world Create and show as modal
			{
				edit_world edit_world1 = new edit_world(this);
				edit_world1.setModal(true);
				edit_world1.show();
			}
		} catch (Exception e) {
		}
	}

	void mainFrame_focusGained(java.awt.event.FocusEvent event)
	{
		// to do: code goes here.
			 
		sphclient_focusGained_Interaction1(event);
	}

	void sphclient_focusGained_Interaction1(java.awt.event.FocusEvent event)
	{
	    
	}


	void aboutItem_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			// about_dialog Create and show as modal
			{
				about_dialog about_dialog1 = new about_dialog(this);
				about_dialog1.setModal(true);
				about_dialog1.show();
			}
		} catch (Exception e) {
		}
	}

	void view_shapes_actionPerformed(java.awt.event.ActionEvent event)
	{
	    view_shapes.setSelected(true);
        view_network.setSelected(false);
        view_density.setSelected(false);
        view_pos.setSelected(false);
        view_pressure.setSelected(false);
        view_vel.setSelected(false);
        view = SHAPE;
        change_view();                
	}

	void view_network_actionPerformed(java.awt.event.ActionEvent event)
	{
	    view_shapes.setSelected(false);
        view_network.setSelected(true);
        view_density.setSelected(false);
        view_pos.setSelected(false);
        view_pressure.setSelected(false);
        view_vel.setSelected(false);
        view = NETWORK;
        change_view();
	}

	void viewPos_actionPerformed(java.awt.event.ActionEvent event)
	{
	    view_shapes.setSelected(false);
        view_network.setSelected(false);        
        view_density.setSelected(false);
        view_pos.setSelected(true);
        view_pressure.setSelected(false);
        view_vel.setSelected(false);
        view = POS;
        change_view();			 
	}


	void viewPressure_actionPerformed(java.awt.event.ActionEvent event)
	{
	    view_shapes.setSelected(false);
        view_network.setSelected(false);        
        view_density.setSelected(false);
        view_pos.setSelected(false);
        view_pressure.setSelected(true);
        view_vel.setSelected(false);
        view = PRESSURE;
        change_view();			 
	}

	void viewDensity_actionPerformed(java.awt.event.ActionEvent event)
	{
	    view_shapes.setSelected(false);
        view_network.setSelected(false);        
        view_density.setSelected(true);
        view_pos.setSelected(false);
        view_pressure.setSelected(false);
        view_vel.setSelected(false);
        view = DENSITY;
        change_view();			 
	}

	void viewVel_actionPerformed(java.awt.event.ActionEvent event)
	{
	    view_shapes.setSelected(false);
        view_network.setSelected(false);        
        view_density.setSelected(false);
        view_pos.setSelected(false);
        view_pressure.setSelected(false);
        view_vel.setSelected(true);
        view = VEL;
        change_view();			 
	}
	
	void change_view()
	{
	    // turn on the elements that belong to this view, 
	    // turn off everything else, change the title of
	    // the view

	    if (view == SHAPE)
	    {
	            network_canvas.setVisible(false);
	            animation_canvas.setVisible(false);
	            shape_canvas.setVisible(true);
	            label_upper_right.setVisible(true);
	            label_lower_left.setVisible(true);
                label_view_title.setText("Boundary shapes");	            
                frame_label.setVisible(false);
                frame_scroll.setVisible(false);
                play_button.setVisible(false);
                stop_button.setVisible(false);
                rewind_button.setVisible(false);
                delay_label.setVisible(false);
                delay_field.setVisible(false);
                min_label.setVisible(false);
                max_label.setVisible(false);
	    }
	    else if (view == NETWORK)
	    {	     
	            network_canvas.setVisible(true);
	            network_canvas.update();
	            shape_canvas.setVisible(false);
	            animation_canvas.setVisible(false);
	            label_upper_right.setVisible(false);
	            label_lower_left.setVisible(false);
	            label_view_title.setText("Network");
	            frame_label.setVisible(false);
                frame_scroll.setVisible(false);
                play_button.setVisible(false);
                stop_button.setVisible(false);
                rewind_button.setVisible(false);
                delay_label.setVisible(false);
                delay_field.setVisible(false);               
                min_label.setVisible(false);
                max_label.setVisible(false);
        }
        else if ((view == POS) || (view == DENSITY) ||
                 (view == PRESSURE) || (view == VEL))
        {	     
	            network_canvas.setVisible(false);	            
	            shape_canvas.setVisible(false);
	            animation_canvas.setVisible(true);
	            label_upper_right.setVisible(false);
	            label_lower_left.setVisible(false);
	            
	            switch (view)
                {
	                case POS:
	                    label_view_title.setText("Positions");
	                    break;
	                case DENSITY:
	                    label_view_title.setText("Density");
	                    break;
	                case PRESSURE:
	                    label_view_title.setText("Pressure");
	                    break;
	                case VEL:
	                    label_view_title.setText("Velocity");
	                    break;
                }
	                    	            
	            frame_label.setVisible(true);
                frame_scroll.setVisible(true);                            
                play_button.setVisible(true);
                stop_button.setVisible(true);
                rewind_button.setVisible(true);                
                delay_label.setVisible(true);
                delay_field.setVisible(true);
        
                if (view == POS)
                {
                    min_label.setVisible(false);
                    max_label.setVisible(false);
                }
                else
                {                    
                    min_label.setVisible(true);
                    max_label.setVisible(true);
                }
                
                // get the animation canvas to update itself
                animation_canvas.last_loaded = -1;
                animation_canvas.update();
	    }
	}


	void JMenuItem3_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		JMenuItem3_actionPerformed_Interaction1(event);			 
	}

	void JMenuItem3_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
		try {
			// edit_network Create and show as modal
			{
				edit_network edit_network1 = new edit_network(this);
				edit_network1.setModal(true);
				edit_network1.show();
			}
		} catch (Exception e) {
		}
	}

	void menuConnect_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		menu_connect_actionPerformed_Interaction1(event);
	}

	void menu_connect_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
	    if ((network.server==null) || (network.port==-1))
        {
	        try     
			    {
    				bad_connect_dialog bad_connect_dialog1 = new bad_connect_dialog(this);
				    bad_connect_dialog1.setModal(true);
				    bad_connect_dialog1.error.setText("You must first enter network settings");
				    bad_connect_dialog1.show();
			    }
		    catch (Exception e) {}   
		}
		else
	    if (!network.connect())
	    {
	        // network connection attempt failed
	        try     
			    {
    				bad_connect_dialog bad_connect_dialog1 = new bad_connect_dialog(this);
				    bad_connect_dialog1.setModal(true);
				    bad_connect_dialog1.error.setText("Connection to " +
				        network.server + " on port " + network.port + 
				        " failed.");
				    bad_connect_dialog1.show();
			    }
		    catch (Exception e) {}   
	    }
	    else
	    {
		    try {
    			// menu_connect Disable the JMenuItem
			    menu_connect.setEnabled(false);
		        menu_disconnect.setEnabled(true);					        
		        menu_shutdown.setEnabled(true);
		        menu_update.setEnabled(true);
		        network_canvas.update();
		        menu_runsim.setEnabled(true);
		    } catch (Exception e) {}				
	    }
    }

	void menuDisconnect_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		menu_disconnect_actionPerformed_Interaction1(event);
	}

	void menu_disconnect_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
	    
	    if (!network.disconnect())
	    {
	        // network connection attempt failed
	        try     
			    {
    				bad_connect_dialog bad_connect_dialog1 = new bad_connect_dialog(this);
				    bad_connect_dialog1.setModal(true);
				    bad_connect_dialog1.error.setText("Disconnection to " +
				        network.server + " on port " + network.port + 
				        " failed.");
				    bad_connect_dialog1.show();
			    }
		    catch (Exception e) {}   
	    }
	    else
	    {
		    try {
    			// menu_connect Disable the JMenuItem
			    menu_connect.setEnabled(true);
		        menu_disconnect.setEnabled(false);			
		        menu_shutdown.setEnabled(false);
		        menu_update.setEnabled(false);
		        network_canvas.update();
		        menu_runsim.setEnabled(false);
		    } catch (Exception e) {}				
	    }
	}

	void menuOpenProject_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		menu_open_project_actionPerformed_Interaction1(event);
	}

	void menu_open_project_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
        // load a project file

		openFileDialog.setVisible(true);        
        
        // load the file if one was selected
        if (openFileDialog.getFile()!=null)
        {
            project.load_project(openFileDialog.getDirectory() +
                                    openFileDialog.getFile());
        }               
	}

	void JMenuItem5_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		JMenuItem5_actionPerformed_Interaction1(event);
	}

	void JMenuItem5_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
        // save a project file

		saveFileDialog.setVisible(true);        
        
        // save the file if one was selected
        if (saveFileDialog.getFile()!=null)
        {
            project.write_file(saveFileDialog.getDirectory() +
                                 saveFileDialog.getFile());
            
            // do we write the files here????
        }
    }

	void menu_shutdown_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		menu_shutdown_actionPerformed_Interaction1(event);
	}

	void menu_shutdown_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
	    // tell the server we want them to shutdown completely
	    
	    if (!network.shutdown())
	    {
	        // network connection attempt failed
	        try     
			    {
    				bad_connect_dialog bad_connect_dialog1 = new bad_connect_dialog(this);
				    bad_connect_dialog1.setModal(true);
				    bad_connect_dialog1.error.setText("Shutdown of " +
				        network.server + " on port " + network.port + 
				        " failed.");
				    bad_connect_dialog1.show();
			    }
		    catch (Exception e) {}   
	    }
	    else
	    {
		    try {
    			// menu_connect Disable the JMenuItem
			    menu_connect.setEnabled(true);
		        menu_disconnect.setEnabled(false);	
		        menu_shutdown.setEnabled(false);
		        menu_update.setEnabled(false);
		        network_canvas.update();
		        menu_runsim.setEnabled(false);
		    } catch (Exception e) {}				
	    }

	}

	void menuUpdate_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		menu_update_actionPerformed_Interaction1(event);
        network_canvas.update();	    
	}

	void menu_update_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{
        if (!network.update_stats())
        {
	        // network update stats failed
	        try     
			    {
    				bad_connect_dialog bad_connect_dialog1 = new bad_connect_dialog(this);
				    bad_connect_dialog1.setModal(true);
				    bad_connect_dialog1.error.setText("Stats update from " +
				        network.server + " on port " + network.port + 
				        " failed.");
				    bad_connect_dialog1.show();
			    }
		    catch (Exception e) {}   
        }        
	}

	void mainFrame_windowClosed(java.awt.event.WindowEvent event)
	{
		// to do: code goes here.
			 
		sphclient_windowClosed_Interaction1(event);
	}

	void sphclient_windowClosed_Interaction1(java.awt.event.WindowEvent event)
	{
        // make sure we are disconnected before we exit
		if (network.connected)		
		    network.disconnect();		

		System.out.println("in the window closing function");
		try {
			this.exitApplication();
		} catch (Exception e) {
		}	
	}
	
    void update_menus()
    {
        menu_file1.setVisible(false);
		menu_file2.setVisible(false);
		menu_file3.setVisible(false);
				
		if (project.last_file1.length()>0)
        {
		    menu_file1.setText(project.last_file1);
		    menu_file1.setVisible(true);
		}
		if (project.last_file2.length()>0)
        {
		    menu_file2.setText(project.last_file2);
		    menu_file2.setVisible(true);
		}
		if (project.last_file3.length()>0)
        {
		    menu_file3.setText(project.last_file3);
		    menu_file3.setVisible(true);
		}		
    }

	void menuFile1_actionPerformed(java.awt.event.ActionEvent event)
	{
	    project.load_project(menu_file1.getText());
	    update_canvases();
	}
	
	void menuFile2_actionPerformed(java.awt.event.ActionEvent event)
	{
	    project.load_project(menu_file2.getText());
        update_canvases();
	}

	void menuFile3_actionPerformed(java.awt.event.ActionEvent event)
	{
	    project.load_project(menu_file3.getText());
	    update_canvases();
	}
	
	void update_canvases()
	{
	    shape_canvas.update();
        network_canvas.update();
        animation_canvas.update();
        resize_canvas();
    }

	void menu_runsim_actionPerformed(java.awt.event.ActionEvent event)
	{
		// to do: code goes here.
			 
		menu_runsim_actionPerformed_Interaction1(event);
	}

	void menu_runsim_actionPerformed_Interaction1(java.awt.event.ActionEvent event)
	{               
        if (!network.run_sim())
        {
	        // network update stats failed
	        try     
			    {
    				bad_connect_dialog bad_connect_dialog1 = new bad_connect_dialog(this);
				    bad_connect_dialog1.setModal(true);
				    bad_connect_dialog1.error.setText("Test run on " +
				        network.server + " on port " + network.port + 
				        " failed.");
				    bad_connect_dialog1.show();
			    }
		    catch (Exception e) {}   
        }        

	}


	class SymAdjustment implements java.awt.event.AdjustmentListener
	{
		public void adjustmentValueChanged(java.awt.event.AdjustmentEvent event)
		{
			Object object = event.getSource();
			if (object == frame_scroll)
				frameScroll_adjustmentValueChanged(event);
		}
	}

	void frameScroll_adjustmentValueChanged(java.awt.event.AdjustmentEvent event)
	{
	    animation_canvas.cur_frame = frame_scroll.getValue();

	    animation_canvas.update();	    
	    update_animation_controls();
	}

    void update_animation_controls()
    {
        int num = animation_canvas.num_frames;
        
        String str = new String();

        // change the label associated with our animation canvas

        if (num < 0)            
            str = new String("Frame: -");                            
        else
            str = new String("Frame: " + animation_canvas.cur_frame + "/"
                            + num);

        frame_label.setText(str);              
              
    }


	void playButton_actionPerformed(java.awt.event.ActionEvent event)
	{
		if (playback.isAlive())
		{
		    playback.playit();
		}
		else
		{
		    playback.start();
		    playback.playit();
		}
			 
	}

	void rewindButton_actionPerformed(java.awt.event.ActionEvent event)
	{
	    if (playback.isAlive())
		{
		    playback.rewindit();
		}
		else
		{
		    playback.start();
		    playback.rewindit();
		}			 
	}

	void stopButton_actionPerformed(java.awt.event.ActionEvent event)
	{
		if (playback.isAlive())
		{
		    playback.stopit();
		}
		else
		{
		    playback.start();
		    playback.stopit();
		}
			 
	}

	void delayField_actionPerformed(java.awt.event.ActionEvent event)
	{
		playback.setDelay((new Integer(delay_field.getText())).intValue());			 
	}


	void openSimMenu_actionPerformed(java.awt.event.ActionEvent event)
	{
		try {			
			openFileDialog.setVisible(true);
		} catch (Exception e) {
		}

        // tell the receiver object that we are working with a new
        // set of sim files.
        if (openFileDialog.getFile()!=null)
        {
            network.receiver.load_sim(openFileDialog.getDirectory(),            
                                    openFileDialog.getFile());
        }               
			 
	}

	void menu_rescale_actionPerformed(java.awt.event.ActionEvent event)
	{
		// put up a progress dialog box, receiver will kill it when 
		// it finishes
		try {
            network.receiver.width = canvas_width;
            network.receiver.height = canvas_height;
            
            // fire up our receiver thread
            if (network.receiver.isAlive())
            {
                network.receiver.mode = RESCALE;
                
                animation_canvas.cur_frame = 0;
                animation_canvas.num_frames = 0;
                update_animation_controls();
                
                network.receiver.resume();
            }
            else
            {
                network.receiver.mode = RESCALE;
                network.receiver.start();
            }
		} catch (Exception e) {
		}

	
	}
}
